@@ -17,11 +17,11 @@
 #'   returned value from \code{kable()}.
 #' @param format A character string. Possible values are \code{latex},
 #'   \code{html}, \code{pipe} (Pandoc's pipe tables), \code{simple} (Pandoc's
-#'   simple tables), and \code{rst}. The value of this argument will be
-#'   automatically determined if the function is called within a \pkg{knitr}
-#'   document. The \code{format} value can also be set in the global option
-#'   \code{knitr.table.format}. If \code{format} is a function, it must return a
-#'   character string.
+#'   simple tables), \code{rst}, \code{jira}, and \code{org} (Emacs Org-mode).
+#'   The value of this argument will be automatically determined if the function
+#'   is called within a \pkg{knitr} document. The \code{format} value can also
+#'   be set in the global option \code{knitr.table.format}. If \code{format} is
+#'   a function, it must return a character string.
 #' @param digits Maximum number of digits for numeric columns, passed to
 #'   \code{round()}. This can also be a vector of length \code{ncol(x)}, to set
 #'   the number of digits for individual columns.
@@ -142,7 +142,7 @@ kable = function(
   if (!is.null(align)) align = rep(align, length.out = m)
   if (row.names) {
     x = cbind(' ' = rownames(x), x)
-    if (!is.null(col.names)) col.names = c(' ', col.names)
+    if (!is.null(col.names)) col.names = tail(c(' ', col.names), ncol(x))
     if (!is.null(align)) align = c('l', align)  # left align row names
   }
   n = nrow(x)
@@ -283,8 +283,11 @@ kable_latex = function(
   midrule = getOption('knitr.table.midrule', if (booktabs) '\\midrule' else '\\hline'),
   linesep = if (booktabs) c('', '', '', '', '\\addlinespace') else '\\hline',
   caption = NULL, caption.short = '', table.envir = if (!is.null(caption)) 'table',
-  escape = TRUE
+  escape = TRUE, ...
 ) {
+  # TODO: remove this hack
+  if (xfun::check_old_package('papaja', '0.1.1') && 'added_colnames' %in% names(list(...)))
+    stop('Throw an error for https://github.com/crsh/papaja/issues/563')
   if (!is.null(align <- attr(x, 'align'))) {
     align = paste(align, collapse = vline)
     align = paste0('{', align, '}')
@@ -390,7 +393,7 @@ kable_html = function(
 #' @noRd
 kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ', padding = 0,
                       align.fun = function(s, a) s, rownames.name = '',
-                      sep.head = sep.col, ...) {
+                      sep.head = sep.col, newline = NULL, ...) {
   # when the column separator is |, replace existing | with its HTML entity
   if (sep.col == '|') for (j in seq_len(ncol(x))) {
     x[, j] = gsub('\\|', '&#124;', x[, j])
@@ -408,11 +411,13 @@ kable_mark = function(x, sep.row = c('=', '=', '='), sep.col = '  ', padding = 0
     ifelse(align == 'c', 2, 1)
   }
   l = pmax(l + padding, 3)  # at least of width 3 for Github Markdown
-  s = unlist(lapply(l, function(i) paste(rep(sep.row[2], i), collapse = '')))
+  s = strrep(sep.row[2], l)
   res = rbind(if (!is.na(sep.row[1])) s, cn, align.fun(s, align),
               x, if (!is.na(sep.row[3])) s)
   res = mat_pad(res, l, align)
-  add_mark_col_sep(res, sep.col, sep.head)
+  res = add_mark_col_sep(res, sep.col, sep.head)
+  if (is.character(newline)) res = gsub('\n', newline, res, fixed = TRUE)
+  res
 }
 
 # add column separators to header and body separately
@@ -429,7 +434,7 @@ kable_rst = function(x, rownames.name = '\\', ...) {
 }
 
 # Pandoc's pipe table
-kable_pipe = function(x, caption = NULL, padding = 1, ...) {
+kable_pipe = function(x, caption = NULL, padding = 1, caption.label = 'Table:', ...) {
   if (is.null(colnames(x))) colnames(x) = rep('', ncol(x))
   res = kable_mark(x, c(NA, '-', NA), '|', padding, align.fun = function(s, a) {
     if (is.null(a)) return(s)
@@ -440,7 +445,7 @@ kable_pipe = function(x, caption = NULL, padding = 1, ...) {
     s
   }, ...)
   res = sprintf('|%s|', res)
-  kable_pandoc_caption(res, caption)
+  kable_pandoc_caption(res, caption, caption.label)
 }
 
 # Pandoc's simple table
@@ -466,9 +471,20 @@ kable_jira = function(x, caption = NULL, padding = 1, ...) {
   kable_pandoc_caption(tab, caption)
 }
 
-kable_pandoc_caption = function(x, caption) {
+# Emacs Org-mode table
+kable_org = function(...) {
+  res = kable_pipe(..., caption.label = '#+CAPTION:')
+  i = grep('^[-:|]+$', res)  # find the line like |--:|---| under header
+  if (length(i)) {
+    i = i[1]
+    res[i] = gsub('(-|:)[|](-|:)', '\\1+\\2', res[i])  # use + as separator
+  }
+  res
+}
+
+kable_pandoc_caption = function(x, caption, label = 'Table:') {
   if (identical(caption, NA)) caption = NULL
-  if (length(caption)) c(paste('Table:', caption), "", x) else x
+  if (length(caption)) c(paste(label, caption), '', x) else x
 }
 
 # pad a matrix
@@ -490,12 +506,7 @@ pad_width = function(x, width, side) {
     stop("'side' must be 'left', 'right', or 'both'")
   w = width - nchar(x, 'width')
   w1 = floor(w / 2)  # the left half of spaces when side = 'both'
-  s1 = v_spaces(w * (side == 'left') + w1 * (side == 'both'))
-  s2 = v_spaces(w * (side == 'right') + (w - w1) * (side == 'both'))
+  s1 = strrep(' ', pmax(0, w * (side == 'left') + w1 * (side == 'both')))
+  s2 = strrep(' ', pmax(0, w * (side == 'right') + (w - w1) * (side == 'both')))
   paste0(s1, x, s2)
 }
-
-# vectorized over n to generate sequences of spaces
-v_spaces = function(n) {
-  unlist(lapply(n, highr:::spaces))
-}
