@@ -44,6 +44,9 @@ comment_to_var = function(x, varname, pattern, envir) {
   FALSE
 }
 
+# TODO: remove this when we don't support R < 3.5.0
+if (getRversion() < '3.5.0') isFALSE = function(x) identical(x, FALSE)
+
 is_blank = function(x) {
   if (length(x)) all(grepl('^\\s*$', x)) else TRUE
 }
@@ -88,9 +91,9 @@ set_preamble = function(input, patterns = knit_patterns$get()) {
   idx1 = grep(hb, input)[1]
   if (is.na(idx1) || idx1 >= idx2) return()
   txt = one_string(input[idx1:(idx2 - 1L)])  # rough preamble
-  idx = stringr::str_locate(txt, hb)  # locate documentclass
-  options(tikzDocumentDeclaration = stringr::str_sub(txt, idx[, 1L], idx[, 2L]))
-  preamble = pure_preamble(split_lines(stringr::str_sub(txt, idx[, 2L] + 1L)), patterns)
+  idx = str_locate(txt, hb, FALSE)  # locate documentclass
+  options(tikzDocumentDeclaration = substr(txt, idx[, 1L], idx[, 2L]))
+  preamble = pure_preamble(split_lines(substr(txt, idx[, 2L] + 1L, nchar(txt))), patterns)
   .knitEnv$tikzPackages = c(.header.sweave.cmd, preamble, '\n')
   .knitEnv$bibliography = grep('^\\\\bibliography.+', input, value = TRUE)
 }
@@ -240,6 +243,26 @@ tikz_dict = function(path) {
   paste(sans_ext(basename(path)), 'tikzDictionary', sep = '-')
 }
 
+# convert dashes in option names with dots (e.g., `fig-height` to `fig.height`)
+dot_names = function(x) {
+  dashes = grep('-', names(x), value = TRUE)
+  dots   = gsub('-', '.', dashes)
+  # only convert names that are known to knitr
+  i = dots %in% c(names(opts_chunk_attr), names(opts_chunk$get()))
+  if (any(i)) {
+    x[dots[i]] = x[dashes[i]]
+    x[dashes[i]] = NULL
+  }
+
+  # normalize aliases (introduced by Quarto)
+  aliases = c(fig.format = 'dev', fig.dpi = 'dpi')
+  for (j in intersect(names(x), names(aliases))) {
+    x[[aliases[j]]] = x[[j]]
+    x[[j]] = NULL
+  }
+  x
+}
+
 # initially for compatibility with Sweave and old beta versions of knitr
 # but now also place to tweak default options
 fix_options = function(options) {
@@ -364,7 +387,7 @@ fix_options = function(options) {
 #' # Test if current output format is 'docx'
 #' knitr::pandoc_to('docx')
 is_latex_output = function() {
-  out_format('latex') || pandoc_to(c('latex', 'beamer'))
+  out_format(c('latex', 'sweave', 'listings')) || pandoc_to(c('latex', 'beamer'))
 }
 
 #' @param fmt A character vector of output formats to be checked against. If not
@@ -376,6 +399,7 @@ is_latex_output = function() {
 #' @rdname output_type
 #' @export
 is_html_output = function(fmt = pandoc_to(), excludes = NULL) {
+  fmt = fmt %n% out_format()
   if (length(fmt) == 0) return(FALSE)
   if (grepl('^markdown', fmt)) fmt = 'markdown'
   if (fmt == 'epub3') fmt = 'epub'
@@ -483,24 +507,17 @@ fig_path = function(suffix = '', options = opts_current$get(), number) {
   if (missing(number)) number = options$fig.cur %n% 1L
   if (!is.null(number)) suffix = paste0('-', number, suffix)
   path = valid_path(options$fig.path, options$label)
-  (if (out_format(c('latex', 'sweave', 'listings'))) sanitize_fn else
-    paste0)(path, suffix)
+  if (getOption('knitr.sanitize.paths', out_format(c('latex', 'sweave', 'listings'))))
+    path = sanitize_fn(path)
+  paste0(path, suffix)
 }
 # sanitize filename for LaTeX
-sanitize_fn = function(path, suffix = '') {
-  if (grepl('[^~:_./\\[:alnum:]-]', path)) {
-    warning('replaced special characters in figure filename "', path, '" -> "',
-            path <- gsub('[^~:_./\\[:alnum:]-]', '_', path), '"')
-  }
-  # replace . with _ except ../ and ./
-  s = strsplit(path, '[/\\\\]')[[1L]]
-  i = (s != '.') & (s != '..') & grepl('\\.', s)
-  if (any(i)) {
-    s[i] = gsub('\\.', '_', s[i])
-    path = paste(s, collapse = '/')
-    warning('dots in figure paths replaced with _ ("', path, '")')
-  }
-  paste0(path, suffix)
+sanitize_fn = function(path, warn = TRUE) {
+  p = gsub('[^~:_./\\[:alnum:]-]', '_', path)
+  if (warn && (p != path)) warning(
+    "Replaced special characters in figure filename '", path, "' -> '", p, "'"
+  )
+  p
 }
 
 #' Obtain the figure filenames for a chunk
@@ -577,7 +594,13 @@ print_knitlog = function() {
 }
 
 # count the number of lines
-line_count = function(x) stringr::str_count(x, '\n') + 1L
+line_count = function(x) {
+  n = lengths(strsplit(x, '\n', fixed = TRUE))
+  i = grep('\n$', x)
+  n[i] = n[i] + 1L  # add an extra count for lines ending with \n
+  n[n == 0] = 1L  # should be at least one line
+  n
+}
 
 has_package = function(pkg) loadable(pkg, FALSE)
 has_packages = function(pkgs) {
@@ -770,16 +793,37 @@ kpsewhich = function() {
 
 # check if a utility exists; if it does, save its availability in opts_knit
 has_utility = function(name, package = name) {
-  name2 = paste('util', name, sep = '_')  # e.g. util_pdfcrop
+  name2 = paste0('util_', name)  # e.g. util_pdfcrop
   if (is.logical(yes <- opts_knit$get(name2))) return(yes)
   # a special case: use tools::find_gs_cmd() to find ghostscript
-  if (name == 'ghostscript') name = tools::find_gs_cmd()
-  yes = nzchar(Sys.which(name))
-  if (!yes) warning(package, ' not installed or not in PATH')
+  if (name == 'pdfcrop') yes = has_crop_tools() else {
+    yes = nzchar(Sys.which(name))
+    if (!yes) warning(package, ' not installed or not in PATH')
+  }
   opts_knit$set(setNames(list(yes), name2))
   yes
 }
 
+# check if pdfcrop and ghostscript are available
+has_crop_tools = function(warn = TRUE) {
+  if (Sys.which('pdfcrop') == '') {
+    if (warn) warning("'pdfcrop' is required but not found")
+    return(FALSE)
+  }
+  if (is_windows() && Sys.which('tlmgr') != '') {
+    # assuming users know what this env var means (rstudio/tinytex#391)
+    if (Sys.getenv('TEXLIVE_WINDOWS_EXTERNAL_GS') != '') return(TRUE)
+    year = tinytex::tlmgr_version('list')$texlive
+    if (year < 2023 && warn) warning(
+      'TeX Live version too low. Please consider upgrading, e.g., via tinytex::reinstall_tinytex().'
+    )
+    return(year >= 2023)
+  }
+  gs = tools::find_gs_cmd() != ''
+  if (warn && !gs) warning("'ghostscript' is required but not found")
+  gs
+}
+
 #' Query the current input filename
 #'
 #' Returns the name of the input file passed to \code{\link{knit}()}.
@@ -1076,12 +1120,64 @@ remove_urls = function(x) {
   gsub("(?<!`)\\[([^]]+)\\]\\(([^)]+)\\)(?!`)", "\\1", x, perl = TRUE)
 }
 
-# repeat a string for n times
-rep_str = function(x, n, sep = '') paste(rep(x, n), collapse = sep)
-
 # patch strsplit() to split '' into '' instead of character(0)
 str_split = function(x, split, ...) {
   y = strsplit(x, split, ...)
   y[x == ''] = list('')
   y
 }
+
+# default progress bar function in knitr: create a text progress bar, and return
+# methods to update/close it
+txt_pb = function(total, labels) {
+  if (getOption('knitr.progress.linenums', FALSE)) {
+    nums = sapply(seq_along(labels), function(i) {
+      paste(current_lines(i), collapse = '-')
+    })
+    labels = sprintf(
+      '%s%s%s:%s', labels, ifelse(labels == '', '', ' @ '),
+      knit_concord$get('infile'), nums
+    )
+  }
+  s = ifelse(labels == '', '', sprintf(' [%s]', labels))  # chunk labels in []
+  w = nchar(s)  # widths of labels
+  n = max(w)
+  # right-pad spaces for same width of all labels so a wider label of the
+  # progress bar in a previous step could be completely wiped (by spaces)
+  s = paste0(s, strrep(' ', n - w))
+  w2 = getOption('width')
+  con = getOption('knitr.progress.output', '')
+  cat_line = function(...) cat(..., sep = '', file = con)
+  # test if it is a "terminal" connection (whether \r is supported)
+  simple = (function() {
+    # a global option to decide whether to use the simple progress output
+    if (!is.null(res <- getOption('knitr.progress.simple'))) return(res)
+    if (identical(con, '')) con = stdout()
+    if (!inherits(con, 'connection')) return(TRUE)
+    if (isatty(con)) return(FALSE)
+    # RStudio's background jobs
+    if (Sys.getenv('RSTUDIO_CHILD_PROCESS_PANE') %in% c('job', 'build')) return(FALSE)
+    # when RStudio is available, return FALSE
+    is.null(tryCatch(rstudioapi::versionInfo(), error = function(e) NULL))
+  })()
+  # use simple progress output without the bar but only progress and labels
+  if (simple) return(list(
+    update = function(i) cat_line(i, '/', total, s[i], '\n')
+  ))
+  pb = txtProgressBar(
+    0, total, 0, '.', width = max(w2 - 10 - n, 10), style = 3, file = con
+  )
+  list(
+    update = function(i) {
+      setTxtProgressBar(pb, i)
+      cat_line(s[i])  # append chunk label to the progress bar
+    },
+    done = function() {
+      # wipe the progress bar
+      cat_line('\r', strrep(' ', max(w2, 10) + 10 + n))
+      close(pb)
+    }
+  )
+}
+
+is_quarto = function() isTRUE(.knitEnv$is_quarto)
